/*
 * proximity efect - two wires in return circuit
 * 2016-08-18 Kouichi Hirabayashi
 * 2017-05-31 Zac η׻
 * 2017-07-02 Zac η׻
 * Cf.
 *   H.B.DWEIGHT,- Proximity Effect in Wires and Thin Tubes
 *	A.I.E.E. Trans. vol 42, 1923
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <complex.h>
#include "Complex.h"

#define COMPLEX double complex

double PI;

// (n + m - 1)! / ((n - 1)! * m!)
//     m
//  = (n - 1 + k) / k
//   k = 1
// ˤʤ餷 (ñʾפդʤ)
double Fnm(int n, int m)
{
  int k;
  double v;

  v = 1.0;
  for (k = 1; k <= m; k++)
	v *= (double)(n - 1 + k) / (double)k;
  return v;
}

COMPLEX A0(double a, COMPLEX A)
{
  COMPLEX z;

  z = I * A * a;
  return 0.5 * z / Jn(1.0, z);
}

COMPLEX An(int n, double a, double s, COMPLEX A)
{
  double d, k;
  COMPLEX z;

  d = (double)n;
  k = pow(a / s, d);
  z = -k * A / Jn(d - 1.0, A);
  return z;
}

COMPLEX Bn(int n, double a, double s, COMPLEX A)
{
  int i, j, k;
  double d, f;
  COMPLEX z, z1;

  d = (double)n;
  z = (COMPLEX)0.0;
  for (k = 1; k < 100; k++) {
	z1 = Jn((double)(k + 1), A) / Jn((double)(n - 1), A);
	z1 *= An(k, a, s, A) * pow(a / s, (double)(n + k)) * Fnm(n, k);
	z += z1;
	if (cabs(z1) / cabs(z) < 1e-6)
		break;
  }
  if (k >= 100)
	fprintf(stderr, "! Bn(%d %g %g) k(%d)\n", n, a, s, k);
  return -z;
}

COMPLEX Cn(int n, double a, double s, COMPLEX A)
{
  int i, j, k;
  double d, f;
  COMPLEX z, z1;

  d = (double)n;
  z = (COMPLEX)0.0;
  for (k = 1; k < 100; k++) {
	z1 = Jn((double)(k + 1), A) / Jn((double)(n - 1), A);
	z1 *= Bn(k, a, s, A) * pow(a / s, (double)(n + k)) * Fnm(n, k);
	z += z1;
	if (cabs(z1) / cabs(z) < 1e-6)
		break;
  }
  if (k >= 100)
	fprintf(stderr, "! Cn(%d %g %g) k(%d)\n", n, a, s, k);
  return -z;
}

COMPLEX Dn(int n, double a, double s, COMPLEX A)
{
  int i, j, k;
  double d, f;
  COMPLEX z, z1;

  d = (double)n;
  z = (COMPLEX)0.0;
  for (k = 1; k < 100; k++) {
	z1 = Jn((double)(k + 1), A) / Jn((double)(n - 1), A);
	z1 *= Cn(k, a, s, A) * pow(a / s, (double)(n + k)) * Fnm(n, k);
	z += z1;
	if (cabs(z1) / cabs(z) < 1e-6)
		break;
  }
  if (k >= 100)
	fprintf(stderr, "! Dn(%d %g %g) k(%d)\n", n, a, s, k);
  return -z;
}

COMPLEX En(int n, double a, double s, COMPLEX A)
{
  int i, j, k;
  double d, f;
  COMPLEX z, z1;

  d = (double)n;
  z = (COMPLEX)0.0;
  for (k = 1; k < 100; k++) {
	z1 = Jn((double)(k + 1), A) / Jn((double)(n - 1), A);
	z1 *= Dn(k, a, s, A) * pow(a / s, (double)(n + k)) * Fnm(n, k);
	z += z1;
	if (cabs(z1) / cabs(z) < 1e-6)
		break;
  }
  if (k >= 100)
	fprintf(stderr, "! Dn(%d %g %g) k(%d)\n", n, a, s, k);
  return -z;
}

COMPLEX UV(int n, COMPLEX A)
{
  COMPLEX z0, z1;
  z0 = Jn(n, A);
  z1 = I * csqrt(I) * Jnd(n, A);
  return Jn(n, A) * I * conj(z1);
}

// Solid wire skin effect (Zac/Rdc)

COMPLEX Solid(double f, double a, double u, double g)
{
  double rdc, x;
  COMPLEX A, z;

  rdc = 1.0 / (g * PI * a * a);
  A = a * csqrt(I * 2.0 * PI * f * u * g);
  x = cabs(A);
  if (x < 1e-6)
	z = (COMPLEX)1.0;
  else if (5e2 < x)
	z = A * 0.5;
  else
	z = In(0, A) * (A / In(1, A)) * 0.5;
  return z;
}

// Zac/Rdc = (Rac + j*Xac)/Rdc

COMPLEX Zac(double a, double s, double g, double u, double f)
{
  static COMPLEX (*fun[])(int, double, double, COMPLEX) = {
	(void *)0, An, Bn, Cn, Dn, En };
  int i, k;
  double b, w;
  COMPLEX A0, A, z0, z, z1, z2;

  // skin effect
  w = 2.0 * PI * f;
  b = a * sqrt(w * u * g);
  A = I * csqrt(I) * b;
  A0 = 0.5 * b * I * csqrt(I) / Jn(1, A);
  z = 2.0 * A0 * conj(A0) * UV(0, A) / b;

  // proximity effect
  for (k = 1; k <= 100; k++) {
	z1 = 0;
	for (i = 1; i <= 4; i++) {
		z2 = (*fun[i])(k, a, s, A);
		z1 += z2;
		if (cabs(z2) / cabs(z1) < 1e-3)
			break;
	}
	z1 = z1 * conj(z1) * (UV(k, A) / b);
	z += z1;
	if (cabs(z1) / cabs(z) < 1e-4)
		break;
  }
  return z;
}

// current density,  r = radius (m),  t = angle (rad)

COMPLEX Irt(double a, double s, double g, double u, double w, double r,
	double t)
{
  static COMPLEX (*f[])(int, double, double, COMPLEX) = {
	(void *)0, An, Bn, Cn, Dn, En };
  int i, k;
  double b;
  COMPLEX A0, A, I1, I2, z, z1, z2;
  COMPLEX i1;

  b = a * sqrt(w * u * g);
  A = I * csqrt(I) * b;
  z = I * A * a;
  A0 = 0.5 * b * I * csqrt(I) / Jn(1, A);
  // total currentr = 1 ȤƤ
  i1 = A0 * Jn(0, I * A * r) / (PI * a * a);
  for (k = 1; k <= 100; k++) {
	z1 = (COMPLEX)0.0;
	for (i = 1; i <= 5; i++) {
		z2 = (*f[i])(k, a, s, A);
		z1 += z2;
		if (cabs(z2) / cabs(z1) < 1e-3)
			break;
	}
	z1 *= Jn(k, z * r) * cos((double)k * t);
	i1 += z1;
	if (cabs(z1) / cabs(i1) < 1e-4)
		break;
  }
  return i1;
}

// proximity effect correction factor

double P(double a, double s, double g, double u, double f)
{
  double p, rdc;
  COMPLEX z, z1;

  rdc = 1.0 / (PI * a * a * g);
  z = Zac(a, s, g, u, f);
  z1 = Solid(f, a, u, g);
  return creal(z) / creal(z1);
}

zac(double f1, double f2)
{
  double a, f, s, u, g, p;
  COMPLEX z, z1, z2;

  a = 2.5e-4;
  s = 1.0e-3;
  u = 4e-7 * PI;
  g = 5.80e7;
  for (f = f1; f <= f2; f *= 1.05) {
	z = Zac(a, s, g, u, f);
	printf("%g %g %g\n", f, creal(z), cimag(z));
  }
  return 0;
}

proximity(double f1, double f2)
{
  double a, f, s, u, g, p;

  a = 2.5e-4;
  s = 1.0e-3;
  u = 4e-7 * PI;
  g = 5.80e7;
  for (f = f1; f <= f2; f *= 1.05) {
	p = P(a, s, g, u, f);
	printf("%g %g\n", f, p);
  }
  return 0;
}

inductance(double f1, double f2)
{
  double a, f, s, u, g, w, rdc;
  COMPLEX z;

  a = 2.5e-4;
  s = 1.0e-3;
  u = 4e-7 * PI;
  g = 5.80e7;
  rdc = PI * a * a * g;
  for (f = f1; f <= f2; f *= 1.05) {
	w = 2.0 * PI * f;
	z = Zac(a, s, g, u, f);
	printf("%g %g\n", f, cimag(z) * rdc / w);
  }
  return 0;
}

Idensity(double a, double s, double g, double u, double f, double r)
{
  double w, t;
  COMPLEX z;

  w = 2.0 * PI * f;
  for (t = 0; t <= 2.0 * PI; t += PI * 0.005) {
	z = Irt(a, s, g, u, w, r, t);
//	printf("%g %g %g\n", t, cabs(z), carg(z));
	printf("%g %g\n", t, cabs(z));
  }
  return 0;
}

int main(void)
{
  double a, s, g, u, r;

  PI = atan2(0.0, -1.0);
  a = 2.5e-4;		// conductor radius (m)
  s = 1.0e-3;		// cinductor spacing (m)  2*a < s
  g = 5.80e7;		// conductivity
  u = 4e-7 * PI;	// magnetic permeability
  zac(1e2, 1e9);	// Zac/Rdc
//inductance(1e3, 1e9);	// Li
// proximity(1e3, 1e9);	// P
// Idensity(a, s, g, u, 1e5, a);	// current density
  exit(0);
}
